@@ -58,7 +58,7 @@
 
 /* This needs to come after some library #include
    to get __GNU_LIBRARY__ defined.  */
-#if defined (__GNU_LIBRARY__) || defined (__CYGWIN__) || defined(DJGPP)
+#if defined (__GNU_LIBRARY__) || defined (__CYGWIN__) || defined(__DJGPP__)
 /* Don't include stdlib.h for
  * non-GNU C libraries
  * non-Cygwin
